"""This module provides reusable UI components.
"""
from europi import CHAR_HEIGHT, CHAR_WIDTH, b1, k1, oled


class Menu:
    """A class representing a menu displayable on the EuroPi screen. The user can select a menu item using the
    configured knob and select it with the configured button(s).

    :param items: a list of the menu items
    :param select_func: the function to call when a menu item is chosen. The function will be called with single argument, the selected item.
    :param select_knob: the knob used to select the menu item, defaults to k1
    :param choice_buttons: a List of Buttons that can be pressed to choose the selected item. Defaults to b1
    """

    def __init__(self, items, select_func, select_knob=k1, choice_buttons=None):
        self.items = items
        self.items.append("----- MENU -----")
        self.select_func = select_func
        self.select_knob = select_knob
        choice_buttons = choice_buttons or [b1]

        # init handlers
        def select():
            if self.selected != len(self.items) - 1:  # ignore the '-- MENU --' item
                self.select_func(self.items[self.selected])

        for b in choice_buttons:
            b.handler_falling(select)

    @property
    def selected(self):
        """The currently selected menu item."""
        return self.select_knob.read_position(steps=len(self.items) - 1)

    def _inverted_text(self, s, x, y):
        """displays the given text with an inverted background"""
        oled.fill_rect(x, y-1, CHAR_WIDTH * len(s), CHAR_HEIGHT+2, 1)
        oled.text(s, x, y, 0)

    def draw_menu(self):
        """This function should be called by your script's main loop in order to display and refresh the menu."""
        current = self.selected
        oled.fill(0)
        oled.text(f"{self.items[current - 1]}", 2, 3, 1)
        self._inverted_text(f"{self.items[current]}", 2, 13)
        if current != len(self.items) - 2:
            # don't show the title at the bottom of the menu
            oled.text(f"{self.items[current + 1]}", 2, 23, 1)
        oled.show()



import os
import time
from collections import OrderedDict

import machine

import europi
from europi import CHAR_HEIGHT, CHAR_WIDTH, Button, reset_state
from europi_script import EuroPiScript

from ui import Menu

SCRIPT_DIR = "/lib/contrib/"
REG_FILE_CODE = 0x8000


class BootloaderMenu(EuroPiScript):
    """A Menu which allows the user to select and execute one of the scripts available to this EuroPi. The menu will
    include the ``EuroPiScripts`` provided to it in the constructor. Button press handlers are added that allow the user
    to exit the script and return to the menu by holding both buttons down for a short period of time. If the module is
    restarted while a script is running it will boot right back into that same script.

    In the menu:

    * **Button 1:** choose the selected item
    * **Button 2:** choose the selected item
    * **Knob 1:** unused
    * **Knob 2:** change the current selection

    In a program that was launched from the menu:

    * Hold both buttons for at least 0.5s and release to return to the menu.

    :param script_classes: a list of Classes implementing EuroPiScript to be included in the menu
    """

    def __init__(self, script_classes):
        self.scripts_config = BootloaderMenu._build_scripts_config(script_classes)
        self.menu = Menu(
            items=list(sorted(self.scripts_config.keys())),
            select_func=self.launch,
            select_knob=europi.k2,
            choice_buttons=[europi.b1, europi.b2],
        )
        self.run_request = None

    @classmethod
    def _is_europi_script(cls, c):
        return issubclass(c, EuroPiScript)

    @staticmethod
    def _build_scripts_config(classes):
        return OrderedDict([(cls.display_name(), cls) for cls in classes if BootloaderMenu._is_europi_script(cls)])

    def launch(self, selected_item):
        self.run_request = selected_item

    def exit_to_menu(self):
        self.remove_state()
        # Attempt to save the state of this script if it has been implemented.
        self.save_state()
        machine.reset()  # why doesn't machine.soft_reset() work anymore?

    def main(self):
        state = self.load_state_str()

        if state:
            self.run_request = state

        else:

            # let the user make a selection
            old_selected = -1
            while not self.run_request:
                if old_selected != self.menu.selected:
                    old_selected = self.menu.selected
                    self.menu.draw_menu()
                time.sleep(0.1)
            self.save_state_str(self.run_request)

        # setup the exit handlers, and execute the selection
        reset_state()  # remove menu's button handlers
        europi.b1._handler_both(europi.b2, self.exit_to_menu)
        europi.b2._handler_both(europi.b1, self.exit_to_menu)

        time.sleep(0.25)
        selected_class = self.scripts_config[self.run_request]
        selected_class().main()



import sys

from setuptools import setup

setup(
    name="micropython-europi",
    version="0.5.0",
    description="EuroPi module for MicroPython",
    long_description="""The EuroPi is a fully user reprogrammable EuroRack module based on the Raspberry Pi Pico, which allows users to process inputs and controls to produce outputs based on code written in Python.""",
    url="https://github.com/Allen-Synthesis/EuroPi",
    author="Allen Synthesis",
    author_email="contact@allensynthesis.co.uk",
    license="Apache 2.0",
    py_modules=[
        "bootloader",
        "calibrate",
        "europi_script",
        "europi",
        "ui",
    ],
    install_requires=[],
)


from machine import Pin, ADC, PWM, freq
from time import sleep
from europi import oled, b1, b2
from europi_script import EuroPiScript

class Calibrate(EuroPiScript):

    @classmethod
    def display_name(cls):
        """Push this script to the end of the menu."""
        return "~Calibrate"

    def main(self):
        # Overclock for faster calibration
        freq(250_000_000)


        ain = ADC(Pin(26, Pin.IN, Pin.PULL_DOWN))
        cv1 = PWM(Pin(21))
        usb = Pin(24, Pin.IN)

        def sample():
            readings = []
            for reading in range(256):
                readings.append(ain.read_u16())
            return round(sum(readings)/256)

        def wait_for_voltage(voltage):
            wait_for_b1(0)
            if voltage != 0:
                oled.centre_text(f'Plug in {voltage}V\n\nDone: Button 1')
                wait_for_b1(1)
            else:
                oled.centre_text(f'Unplug all\n\nDone: Button 1')
                wait_for_b1(1)
            oled.centre_text('Calibrating...')
            sleep(1.5)
            return sample()

        def text_wait(text, wait):
            oled.centre_text(text)
            sleep(wait)

        def fill_show(colour):
            oled.fill(colour)
            oled.show()

        def flash(flashes, period):
            for flash in range(flashes):
                fill_show(1)
                sleep(period/2)
                fill_show(0)
                sleep(period/2)

        def wait_for_b1(value):
            while b1.value() != value:
                sleep(0.05)


        # Calibration start

        if usb.value() == 1:
            oled.centre_text('Make sure rack\npower is on\nDone: Button 1')
            wait_for_b1(1)
            wait_for_b1(0)

        text_wait('Calibration\nMode', 3)

        oled.centre_text('Choose Process\n\n1         2')
        while True:
            if b1.value() == 1:
                chosen_process = 1
                break
            elif b2.value() == 1:
                chosen_process = 2
                break


        # Input calibration

        readings = []
        if chosen_process == 1:
            readings.append(wait_for_voltage(0))
            readings.append(wait_for_voltage(10))
        else:
            for voltage in range(11):
                readings.append(wait_for_voltage(voltage))

        with open(f'lib/calibration_values.py', 'w') as file:
            values = ", ".join(map(str, readings))
            file.write(f"INPUT_CALIBRATION_VALUES=[{values}]")


        # Output Calibration

        oled.centre_text(f'Plug CV1 into\nanalogue in\nDone: Button 1')
        wait_for_b1(1)
        oled.centre_text('Calibrating...')

        if chosen_process == 1:
            new_readings = [readings[0]]
            m = (readings[1] - readings[0]) / 10
            c = readings[0]
            for x in range(1,10):
                new_readings.append(round((m * x) + c))
            new_readings.append(readings[1])
            readings = new_readings

        output_duties = [0]
        duty = 0
        cv1.duty_u16(duty)
        reading = sample()
        for index, expected_reading in enumerate(readings[1:]):
            while abs(reading - expected_reading) > 0.002 and reading < expected_reading:
                cv1.duty_u16(duty)
                duty += 10
                reading = sample()
            output_duties.append(duty)
            oled.centre_text(f'Calibrating...\n{index+1}V')

        with open(f'lib/calibration_values.py', 'a+') as file:
            values = ", ".join(map(str, output_duties))
            file.write(f"\nOUTPUT_CALIBRATION_VALUES=[{values}]")


        oled.centre_text('Calibration\nComplete!')

if __name__ == "__main__":
    Calibrate().main()